from motor.motor_asyncio import AsyncIOMotorClient, AsyncIOMotorDatabase
from nonebot import logger
from pymongo.errors import OperationFailure

from nonebot_plugin_pixivbot.config import Config
from nonebot_plugin_pixivbot.data.errors import DataSourceNotReadyError
from nonebot_plugin_pixivbot.data.source.mongo.migration import MongoMigrationManager
from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.utils.lifecycler import on_startup, on_shutdown


@context.register_singleton()
class MongoDataSource:
    conf = context.require(Config)
    app_db_version = 2

    def __init__(self):
        self._client = None
        self._db = None

        on_startup(self.initialize, replay=True)
        on_shutdown(self.finalize)

    @property
    def client(self):
        return self._client

    @property
    def db(self):
        if self._db is not None:
            return self._db
        else:
            raise DataSourceNotReadyError()

    @staticmethod
    async def _get_db_version(db: AsyncIOMotorDatabase) -> int:
        version = await db["meta_info"].find_one({"key": "db_version"})
        if version is None:
            await db["meta_info"].insert_one({"key": "db_version", "value": 1})
            return 1
        else:
            return version["value"]

    @staticmethod
    async def _set_db_version(db_version: int, db: AsyncIOMotorDatabase):
        await db["meta_info"].update_one({"key": "db_version"},
                                         {"$set": {
                                             "value": db_version
                                         }},
                                         upsert=True)

    @staticmethod
    async def _ensure_index(db: AsyncIOMotorDatabase, coll_name: str, indexes: list[tuple[str, int]], **kwargs):
        try:
            await db[coll_name].create_index(indexes, **kwargs)
        except OperationFailure:
            logger.info(f"Index in {coll_name}: recreated")
            await db[coll_name].drop_index(indexes)
            await db[coll_name].create_index(indexes, **kwargs)

    @staticmethod
    async def _ensure_ttl_index(db: AsyncIOMotorDatabase, coll_name: str, expires_in: int):
        try:
            await db[coll_name].create_index([("update_time", 1)], expireAfterSeconds=expires_in)
        except OperationFailure:
            await db.command({
                "collMod": coll_name,
                "index": {
                    "keyPattern": {"update_time": 1},
                    "expireAfterSeconds": expires_in,
                }
            })
            logger.success(f"TTL Index in {coll_name}: expireAfterSeconds changed to {expires_in}")

    async def initialize(self):
        client = AsyncIOMotorClient(self.conf.pixiv_mongo_conn_url)
        db = client[self.conf.pixiv_mongo_database_name]

        # migrate
        db_version = await self._get_db_version(db)
        await context.require(MongoMigrationManager).perform_migration(db, db_version, self.app_db_version)
        await self._set_db_version(self.app_db_version, db)

        # ensure index
        await self._ensure_index(db, 'meta_info', [("key", 1)], unique=True)

        await self._ensure_index(db, 'pixiv_binding', [("adapter", 1), ("user_id", 1)], unique=True)

        await self._ensure_index(db, 'subscription', [("adapter", 1), ("user_id", 1), ("type", 1)])
        await self._ensure_index(db, 'subscription', [("adapter", 1), ("group_id", 1), ("type", 1)])

        await self._ensure_index(db, 'local_tags', [("name", 1)], unique=True)
        await self._ensure_index(db, 'local_tags', [("translated_name", 1)])

        await self._ensure_index(db, 'download_cache', [("illust_id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'download_cache', self.conf.pixiv_download_cache_expires_in)

        await self._ensure_index(db, 'illust_detail_cache', [("illust.id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'illust_detail_cache', self.conf.pixiv_illust_detail_cache_expires_in)

        await self._ensure_index(db, 'user_detail_cache', [("user.id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'user_detail_cache', self.conf.pixiv_user_detail_cache_expires_in)

        await self._ensure_index(db, 'illust_ranking_cache', [("mode", 1)], unique=True)
        await self._ensure_ttl_index(db, 'illust_ranking_cache', self.conf.pixiv_illust_ranking_cache_expires_in)

        await self._ensure_index(db, 'search_illust_cache', [("word", 1)], unique=True)
        await self._ensure_ttl_index(db, 'search_illust_cache', self.conf.pixiv_search_illust_cache_expires_in)

        await self._ensure_index(db, 'search_user_cache', [("word", 1)], unique=True)
        await self._ensure_ttl_index(db, 'search_user_cache', self.conf.pixiv_search_user_cache_expires_in)

        await self._ensure_index(db, 'user_illusts_cache', [("user_id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'user_illusts_cache', self.conf.pixiv_user_illusts_cache_expires_in)

        await self._ensure_index(db, 'user_bookmarks_cache', [("user_id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'user_bookmarks_cache', self.conf.pixiv_user_bookmarks_cache_expires_in)

        await self._ensure_index(db, 'related_illusts_cache', [("original_illust_id", 1)], unique=True)
        await self._ensure_ttl_index(db, 'related_illusts_cache', self.conf.pixiv_related_illusts_cache_expires_in)

        await self._ensure_index(db, 'other_cache', [("type", 1)], unique=True)
        await self._ensure_ttl_index(db, 'other_cache', self.conf.pixiv_other_cache_expires_in)

        self._client = client
        self._db = db
        logger.success("MongoDataSource Initialization Succeed.")

    async def finalize(self):
        self._client.close()
        self._client = None
        self._db = None


__all__ = ("MongoDataSource",)



from abc import ABC, abstractmethod
from typing import Type

from nonebot import logger
from nonebot.adapters import Bot, Event
from nonebot.matcher import Matcher
from nonebot.typing import T_State

from nonebot_plugin_pixivbot.context import Context


class Query(ABC):
    @property
    @abstractmethod
    def matcher(self) -> Matcher:
        raise NotImplementedError()

    @abstractmethod
    async def on_match(self, bot: Bot, event: Event, state: T_State, matcher: Matcher):
        raise NotImplementedError()


def register_query(context: Context):
    def decorator(cls: Type[Query]):
        if cls in context:
            q = context.require(cls)
        else:
            q = cls()
            context.register(cls, q)

        q.matcher.append_handler(q.on_match)
        logger.success(f"registered query {cls}")
        return cls

    return decorator



from abc import ABC
from typing import TypeVar

from nonebot_plugin_pixivbot.global_context import context
from .handler import Handler
from .interceptor.default_error_interceptor import DefaultErrorInterceptor
from .interceptor.permission_interceptor import BlacklistInterceptor

UID = TypeVar("UID")
GID = TypeVar("GID")


class EntryHandler(Handler, ABC):
    def __init__(self):
        super().__init__()
        self.add_interceptor(context.require(DefaultErrorInterceptor))
        self.add_interceptor(context.require(BlacklistInterceptor))



from abc import ABC, abstractmethod
from typing import TypeVar, Dict, Type, Generic

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman.model import IllustMessageModel, IllustMessagesModel
from nonebot_plugin_pixivbot.postman.post_destination import PostDestination

UID = TypeVar("UID")
GID = TypeVar("GID")

PD = PostDestination[UID, GID]


class Postman(ABC, Generic[UID, GID]):
    @classmethod
    @abstractmethod
    def adapter(cls) -> str:
        raise NotImplementedError()

    @abstractmethod
    async def send_plain_text(self, message: str,
                              *, post_dest: PD):
        raise NotImplementedError()

    @abstractmethod
    async def send_illust(self, model: IllustMessageModel,
                          *, post_dest: PD):
        raise NotImplementedError()

    @abstractmethod
    async def send_illusts(self, model: IllustMessagesModel,
                           *, post_dest: PD):
        raise NotImplementedError()


@context.register_singleton()
class PostmanManager:
    def __init__(self):
        self.postmen: Dict[str, Type[Postman]] = {}

    def register(self, cls: Type[Postman]):
        self.postmen[cls.adapter()] = cls
        if cls not in context:
            context.register_singleton()(cls)
        return cls

    def require(self, adapter: str):
        return context.require(self.postmen[adapter])

    def __getitem__(self, adapter: str):
        return self.require(adapter)

    async def send_plain_text(self, message: str,
                              *, post_dest: PD):
        return await self[post_dest.adapter].send_plain_text(message, post_dest=post_dest)

    async def send_illust(self, model: IllustMessageModel,
                          *, post_dest: PD):
        return await self[post_dest.adapter].send_illust(model, post_dest=post_dest)

    async def send_illusts(self, model: IllustMessagesModel,
                           *, post_dest: PD):
        return await self[post_dest.adapter].send_illusts(model, post_dest=post_dest)


__all__ = ("Postman", "PostmanManager")



from abc import ABC
from typing import TypeVar

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman import PostDestination
from nonebot_plugin_pixivbot.service.pixiv_service import PixivService
from .recorder import Req, Recorder
from ..entry_handler import EntryHandler
from ..interceptor.cooldown_interceptor import CooldownInterceptor

UID = TypeVar("UID")
GID = TypeVar("GID")


class CommonHandler(EntryHandler, ABC):
    def __init__(self):
        super().__init__()
        self.service = context.require(PixivService)
        self.recorder = context.require(Recorder)

        self.add_interceptor(context.require(CooldownInterceptor))

    def record_req(self, *args,
                   post_dest: PostDestination[UID, GID],
                   **kwargs):
        self.recorder.record_req(Req(self, *args, **kwargs), post_dest.identifier)

    def record_resp_illust(self, illust_id: int,
                           post_dest: PostDestination[UID, GID]):
        self.recorder.record_resp(illust_id, post_dest.identifier)



from __future__ import annotations

import re
from datetime import datetime, timedelta
from inspect import isawaitable
from typing import TypeVar, Dict, List, Sequence, Union, Optional, TYPE_CHECKING

from apscheduler.schedulers.asyncio import AsyncIOScheduler
from apscheduler.triggers.interval import IntervalTrigger
from lazy import lazy
from nonebot import logger, Bot

from nonebot_plugin_pixivbot.data.subscription_repo import SubscriptionRepo
from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.model import Subscription, PostIdentifier
from nonebot_plugin_pixivbot.postman.post_destination import PostDestination, PostDestinationFactoryManager
from nonebot_plugin_pixivbot.utils.errors import BadRequestError
from nonebot_plugin_pixivbot.utils.lifecycler import on_bot_connect, on_bot_disconnect
from nonebot_plugin_pixivbot.utils.nonebot import get_adapter_name

if TYPE_CHECKING:
    from nonebot_plugin_pixivbot.handler import Handler

UID = TypeVar("UID")
GID = TypeVar("GID")

ID = PostIdentifier[UID, GID]


@context.register_eager_singleton()
class Scheduler:
    def __init__(self):
        self.apscheduler = context.require(AsyncIOScheduler)
        self.subscriptions = context.require(SubscriptionRepo)

        on_bot_connect(self.on_bot_connect, replay=True)
        on_bot_disconnect(self.on_bot_disconnect)

    @staticmethod
    def _make_job_id(type: str, identifier: ID):
        if identifier.group_id:
            return f'scheduled {type} {identifier.adapter}:g{identifier.group_id}'
        else:
            return f'scheduled {type} {identifier.adapter}:u{identifier.user_id}'

    @staticmethod
    def _parse_schedule(raw_schedule: str) -> Sequence[int]:
        start_only_mat = re.fullmatch(r'(\d+):(\d+)', raw_schedule)
        if start_only_mat is not None:
            g = start_only_mat.groups()
            start_hour, start_minute = int(g[0]), int(g[1])
            interval_hour, interval_minute = 24, 0
        else:
            interval_only_mat = re.fullmatch(
                r'(\d+):(\d+)\*x', raw_schedule)
            if interval_only_mat is not None:
                g = interval_only_mat.groups()
                start_hour, start_minute = 0, 0
                interval_hour, interval_minute = int(g[0]), int(g[1])
            else:
                mat = re.fullmatch(r'(\d+):(\d+)\+(\d+):(\d+)\*x', raw_schedule)
                if mat is not None:
                    g = mat.groups()
                    start_hour, start_minute = int(g[0]), int(g[1])
                    interval_hour, interval_minute = int(g[2]), int(g[3])
                else:
                    raise BadRequestError(f'{raw_schedule}不是合法的时间')

        if start_hour < 0 or start_hour >= 24 or start_minute < 0 or start_minute >= 60 \
                or interval_hour < 0 or interval_minute >= 60 \
                or (interval_hour > 0 and interval_minute < 0) or (interval_hour == 0 and interval_minute <= 0):
            raise BadRequestError(f'{raw_schedule}不是合法的时间')

        return start_hour, start_minute, interval_hour, interval_minute

    @lazy
    def _handlers(self) -> Dict[str, Handler]:
        # 解决Handler和Scheduler的循环引用
        from nonebot_plugin_pixivbot.handler.common import RandomBookmarkHandler, RandomRecommendedIllustHandler, \
            RankingHandler, RandomIllustHandler, RandomUserIllustHandler
        return {
            RandomBookmarkHandler.type(): context.require(RandomBookmarkHandler),
            RandomRecommendedIllustHandler.type(): context.require(RandomRecommendedIllustHandler),
            RankingHandler.type(): context.require(RankingHandler),
            RandomIllustHandler.type(): context.require(RandomIllustHandler),
            RandomUserIllustHandler.type(): context.require(RandomUserIllustHandler),
        }

    def _add_job(self, post_dest: PostDestination[UID, GID], sub: Subscription[UID, GID]):
        offset_hour, offset_minute, hours, minutes = sub.schedule
        trigger = IntervalTrigger(hours=hours, minutes=minutes,
                                  start_date=datetime.now().replace(hour=offset_hour, minute=offset_minute,
                                                                    second=0, microsecond=0) + timedelta(days=-1))

        identifier = sub.identifier
        job_id = self._make_job_id(sub.type, identifier)
        self.apscheduler.add_job(self._handlers[sub.type].handle, id=job_id, trigger=trigger,
                                 kwargs={"post_dest": post_dest, "silently": True, **sub.kwargs})
        logger.success(f"scheduled {job_id} {trigger}")

    def _remove_job(self, type: str, identifier: ID):
        job_id = self._make_job_id(type, identifier)
        self.apscheduler.remove_job(job_id)
        logger.success(f"unscheduled {job_id}")

    async def on_bot_connect(self, bot: Bot):
        adapter = get_adapter_name(bot)
        async for sub in self.subscriptions.get_all(adapter):
            post_dest = context.require(PostDestinationFactoryManager).build(bot, sub.user_id, sub.group_id)
            self._add_job(post_dest, sub)

    async def on_bot_disconnect(self, bot: Bot):
        async for subscription in self.subscriptions.get_all(get_adapter_name(bot)):
            try:
                self._remove_job(subscription.type, subscription.identifier)
            except Exception as e:
                logger.error(
                    f"error occurred when remove job {self._make_job_id(subscription.type, subscription.identifier)}")
                logger.exception(e)

    async def schedule(self, type: str,
                       schedule: Union[str, Sequence[int]],
                       args: Optional[list] = None,
                       *, post_dest: PostDestination[UID, GID]):
        if type not in self._handlers:
            raise BadRequestError(f"{type}不是合法的类型")

        if args is None:
            args = []

        if isinstance(schedule, str):
            schedule = self._parse_schedule(schedule)

        kwargs = self._handlers[type].parse_args(args, post_dest)
        if isawaitable(kwargs):
            kwargs = await kwargs

        sub = Subscription(adapter=post_dest.adapter,
                           user_id=post_dest.user_id,
                           group_id=post_dest.group_id,
                           type=type,
                           schedule=schedule,
                           kwargs=kwargs)

        old_sub = await self.subscriptions.update(sub)
        if old_sub is not None:
            self._remove_job(sub.type, sub.identifier)
        self._add_job(post_dest.normalized(), sub)

    async def unschedule(self, type: str,
                         identifier: PostIdentifier[UID, GID]):
        if type == "all":
            async for sub in self.subscriptions.get(identifier):
                self._remove_job(sub.type, identifier)
        elif type in self._handlers:
            self._remove_job(type, identifier)
        else:
            raise BadRequestError(f"{type}不是合法的类型")

        await self.subscriptions.delete(identifier, type)

    async def all_subscription(self, identifier: PostIdentifier[UID, GID]) -> List[dict]:
        return [x async for x in self.subscriptions.get(identifier)]


__all__ = ("Scheduler",)



from .command import CommandQuery
from .delegateion_query import DelegationQuery
from .illust import IllustQuery
from .more import MoreQuery
from .query import Query, register_query
from .random_bookmark import RandomBookmarkQuery
from .random_illust import RandomIllustQuery
from .random_recommended_illust import RandomRecommendedIllustQuery
from .random_related_illust import RandomRelatedIllustQuery
from .random_user_illust import RandomUserIllustQuery
from .ranking import RankingQuery

__all__ = ("CommandQuery", "DelegationQuery", "IllustQuery", "MoreQuery", "Query", "RandomBookmarkQuery", "RandomIllustQuery",
           "RandomRecommendedIllustQuery", "RandomRelatedIllustQuery", "RandomUserIllustQuery", "RankingQuery",
           "register_query")



from abc import ABC, abstractmethod

from nonebot.adapters import Bot, Event
from nonebot.matcher import Matcher
from nonebot.typing import T_State

from .query import Query


class DelegationQuery(Query, ABC):
    @property
    @abstractmethod
    def delegation(self) -> Query:
        raise NotImplementedError()

    async def on_match(self, bot: Bot, event: Event, state: T_State, matcher: Matcher):
        await self.delegation.on_match(bot, event, state, matcher)



from lazy import lazy
from nonebot import on_regex
from nonebot.adapters import Bot, Event
from nonebot.matcher import Matcher
from nonebot.typing import T_State

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.handler.common import RandomUserIllustHandler
from .query import Query, register_query
from .utils import get_count, get_common_query_rule, get_post_dest


@register_query(context)
class RandomUserIllustQuery(Query):
    def __init__(self):
        super().__init__()
        self.handler = context.require(RandomUserIllustHandler)

    @lazy
    def matcher(self):
        return on_regex("^来(.*)?张(.+)老师的图$", rule=get_common_query_rule(), priority=4, block=True)

    async def on_match(self, bot: Bot, event: Event, state: T_State, matcher: Matcher):
        user = state["_matched_groups"][1]

        await self.handler.handle(user, count=get_count(state), post_dest=get_post_dest(bot, event))



from lazy import lazy


class LazyDelegation:
    def __init__(self, builder):
        self.builder = builder

    @lazy
    def by(self):
        return self.builder()

    def __getattr__(self, item):
        return self.by.__getattribute__(item)



"""
nonebot-plugin-pixivbot

@Author         : ssttkkl
@License        : MIT
@GitHub         : https://github.com/ssttkkl/nonebot-plugin-pixivbot
"""

# ================= provide beans =================
from .global_context import context
from .providers import provide

provide(context)

# ================= register query =================
from .query import *

# =============== register scheduler ===============
from .service.scheduler import Scheduler

__all__ = ("context",)



from abc import ABC, abstractmethod
from inspect import isawaitable
from typing import Awaitable, Union, TypeVar, Sequence, Any

from nonebot_plugin_pixivbot.config import Config
from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman.post_destination import PostDestination
from nonebot_plugin_pixivbot.utils.errors import BadRequestError
from .interceptor.combined_interceptor import CombinedInterceptor
from .interceptor.interceptor import Interceptor

UID = TypeVar("UID")
GID = TypeVar("GID")

PD = PostDestination[UID, GID]


class Handler(ABC):
    def __init__(self):
        self.conf = context.require(Config)
        self.interceptor = None

    @classmethod
    @abstractmethod
    def type(cls) -> str:
        raise NotImplementedError()

    @abstractmethod
    def enabled(self) -> bool:
        raise NotImplementedError()

    def parse_args(self, args: Sequence[Any], post_dest: PD) -> Union[dict, Awaitable[dict]]:
        """
        将位置参数转化为命名参数
        :param args: 位置参数sequence
        :param post_dest: PostDestination
        :return: 命名参数dict
        """
        return {}

    async def handle(self, *args,
                     post_dest: PD,
                     silently: bool = False,
                     disabled_interceptors: bool = False,
                     **kwargs):
        if not self.enabled():
            return

        try:
            parsed_kwargs = self.parse_args(args, post_dest)
            if isawaitable(parsed_kwargs):
                parsed_kwargs = await parsed_kwargs
        except Exception as e:
            raise BadRequestError("参数错误") from e

        kwargs = {**kwargs, **parsed_kwargs}

        if self.interceptor is not None:
            await self.interceptor.intercept(self.actual_handle,
                                             post_dest=post_dest,
                                             silently=silently,
                                             disabled_interceptors=disabled_interceptors,
                                             **kwargs)
        else:
            await self.actual_handle(post_dest=post_dest, silently=silently, **kwargs)

    @abstractmethod
    async def actual_handle(self, *, post_dest: PD,
                            silently: bool = False, **kwargs):
        """
        处理指令
        :param post_dest: PostDestination
        :param silently: 失败时不发送消息
        :param kwargs: 参数dict
        """
        raise NotImplementedError()

    def add_interceptor(self, interceptor: Interceptor[UID, GID]):
        if self.interceptor:
            self.interceptor = CombinedInterceptor(
                self.interceptor, interceptor)
        else:
            self.interceptor = interceptor

# class PermissionInterceptorDelegation(PermissionInterceptor, Generic[UID, GID]):
#     def __init__(self):
#         self.delegation = None
#
#     def has_permission(self, post_dest: PD) -> Union[bool, Awaitable[bool]]:
#         if self.delegation:
#             return self.delegation.has_permission(post_dest)
#         else:
#             return True



from typing import TypeVar, Optional

from nonebot_plugin_pixivbot.data.pixiv_binding_repo import PixivBindingRepo
from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.model.pixiv_binding import PixivBinding

UID = TypeVar("UID")


@context.register_singleton()
class PixivAccountBinder:
    def __init__(self):
        self.repo = context.require(PixivBindingRepo)

    async def bind(self, adapter: str, user_id: UID, pixiv_user_id: int):
        binding = PixivBinding(adapter=adapter, user_id=user_id, pixiv_user_id=pixiv_user_id)
        await self.repo.update(binding)

    async def unbind(self, adapter: str, user_id: UID) -> bool:
        return await self.repo.remove(adapter, user_id)

    async def get_binding(self, adapter: str, user_id: UID) -> Optional[int]:
        binding = await self.repo.get(adapter, user_id)
        if binding:
            return binding["pixiv_user_id"]
        else:
            return None



from .identifier import UserIdentifier, PostIdentifier
from .illust import Illust
from .pixiv_binding import PixivBinding
from .subscription import Subscription
from .tag import Tag
from .user import User

__all__ = ("Illust", "User", "Tag", "Subscription", "UserIdentifier", "PostIdentifier", "PixivBinding")



from abc import ABC
from abc import abstractmethod
from typing import Type, Callable, Union, Awaitable, TypeVar, Sequence, Any

from nonebot import logger

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman import PostDestination
from nonebot_plugin_pixivbot.utils.errors import BadRequestError
from ..entry_handler import EntryHandler
from ..handler import Handler
from ..utils import post_plain_text

UID = TypeVar("UID")
GID = TypeVar("GID")


class SubCommandHandler(Handler, ABC):
    @abstractmethod
    def parse_args(self, args: Sequence[Any], post_dest: PostDestination[UID, GID]) -> Union[dict, Awaitable[dict]]:
        raise NotImplementedError()

    async def handle_bad_request(self, *, post_dest: PostDestination[UID, GID],
                                 silently: bool = False,
                                 err: BadRequestError):
        if self.interceptor is not None:
            await self.interceptor.intercept(self.actual_handle_bad_request,
                                             post_dest=post_dest, silently=silently,
                                             err=err)
        else:
            await self.actual_handle_bad_request(post_dest=post_dest, silently=silently,
                                                 err=err)

    async def actual_handle_bad_request(self, *, post_dest: PostDestination[UID, GID],
                                        silently: bool = False,
                                        err: BadRequestError):
        if not silently:
            await post_plain_text(err.message, post_dest=post_dest)


@context.register_singleton()
class CommandHandler(EntryHandler):
    def __init__(self):
        super().__init__()
        self.handlers = dict[str, Type[SubCommandHandler]]()

    @classmethod
    def type(cls) -> str:
        return "command"

    def enabled(self) -> bool:
        return True

    def sub_command(self, type: str) \
            -> Callable[[Type[SubCommandHandler]], Type[SubCommandHandler]]:
        def decorator(cls: Type[SubCommandHandler]):
            if cls not in context:
                context.register_singleton()(cls)
            self.handlers[type] = cls
            logger.success(f"registered subcommand {type}")
            return cls

        return decorator

    def parse_args(self, args: Sequence[Any], post_dest: PostDestination[UID, GID]) -> dict:
        return {"args": args[0]}

    async def actual_handle(self, *, args: Sequence[Any],
                            post_dest: PostDestination[UID, GID],
                            silently: bool = False):
        logger.debug("args: " + " ".join(map(str, args)))
        if len(args) == 0:
            handler = context.require(self.handlers["help"])
        elif args[0] not in self.handlers:
            if not silently:
                await post_plain_text(f"不存在命令 '{args[0]}'", post_dest=post_dest)
            return
        else:
            handler = context.require(self.handlers[args[0]])

        try:
            await handler.handle(*args[1:], post_dest=post_dest)
        except BadRequestError as e:
            await handler.handle_bad_request(err=e, post_dest=post_dest, silently=silently)



from typing import Callable, TypeVar, Generic

from nonebot import logger

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman import PostDestination
from nonebot_plugin_pixivbot.utils.errors import BadRequestError, QueryError
from .interceptor import Interceptor
from ..utils import post_plain_text

UID = TypeVar("UID")
GID = TypeVar("GID")


@context.register_singleton()
class DefaultErrorInterceptor(Interceptor[UID, GID], Generic[UID, GID]):
    async def actual_intercept(self, wrapped_func: Callable, *,
                               post_dest: PostDestination[UID, GID],
                               silently: bool,
                               **kwargs):
        try:
            await wrapped_func(post_dest=post_dest, silently=silently, **kwargs)
        except TimeoutError:
            logger.warning("Timeout")
            if not silently:
                await post_plain_text(f"下载超时", post_dest=post_dest)
        except BadRequestError as e:
            if not silently:
                await post_plain_text(str(e), post_dest=post_dest)
        except QueryError as e:
            if not silently:
                await post_plain_text(str(e), post_dest=post_dest)
        except Exception as e:
            logger.exception(e)
            if not silently:
                await post_plain_text(f"内部错误：{type(e)}{e}", post_dest=post_dest)



from abc import ABC, abstractmethod
from typing import TypeVar, Union, Awaitable

from nonebot_plugin_pixivbot.postman import PostDestination

UID = TypeVar("UID")
GID = TypeVar("GID")


class UserAuthenticator(ABC):
    @abstractmethod
    def group_admin(self, post_dest: PostDestination[UID, GID]) -> Union[bool, Awaitable[bool]]:
        raise NotImplementedError()



from typing import TypeVar

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.postman import PostDestination
from nonebot_plugin_pixivbot.utils.errors import BadRequestError
from .common import CommonHandler

UID = TypeVar("UID")
GID = TypeVar("GID")


@context.root.register_singleton()
class MoreHandler(CommonHandler):
    @classmethod
    def type(cls) -> str:
        return "more"

    def enabled(self) -> bool:
        return self.conf.pixiv_more_enabled

    async def actual_handle(self, *, count: int = 1,
                            post_dest: PostDestination[UID, GID],
                            silently: bool = False):
        req = self.recorder.get_req(post_dest.identifier)
        if not req:
            raise BadRequestError("你还没有发送过请求")

        await req(count=count, post_dest=post_dest, disabled_interceptors=True)



from threading import Lock
from typing import TypeVar, Type, Callable

from nonebot.log import logger

T = TypeVar("T")


class Context:
    def __init__(self, parent=None):
        self._parent = parent
        self._container = {}
        self._lazy_container = {}
        self._binding = {}
        self._lock = Lock()

    @property
    def parent(self):
        return self._parent

    @property
    def root(self):
        if self._parent is None:
            return self
        else:
            return self._parent.root

    def register(self, key: Type[T], bean: T):
        """
        register a bean
        """
        self._container[key] = bean
        logger.success(f"registered bean {key}")

    def register_lazy(self, key: Type[T], bean_initializer: Callable[[], T]):
        """
        register a bean lazily
        """
        if key in self._container:
            del self._container[key]
        self._lazy_container[key] = bean_initializer
        logger.success(f"lazily registered bean {key}")

    def register_singleton(self, *args, **kwargs):
        """
        decorator for a class to register a bean lazily
        """

        def decorator(cls):
            self.register_lazy(cls, lambda: cls(*args, **kwargs))
            return cls

        return decorator

    def register_eager_singleton(self, *args, **kwargs):
        """
        decorator for a class to register a bean lazily
        """

        def decorator(cls):
            bean = cls(*args, **kwargs)
            self.register(cls, bean)
            return cls

        return decorator

    def unregister(self, key: Type[T]):
        """
        unregister the bean of key
        """
        if key in self._container:
            del self._container[key]
        if key in self._lazy_container:
            del self._container[key]

    def bind_to(self, key, src_key):
        """
        bind key (usually the implementation class) to src_key (usually the base class)
        """
        self._binding[key] = src_key
        logger.success(f"bind bean {key} to {src_key}")

    def bind_singleton_to(self, key, *args, **kwargs):
        """
        decorator for a class (usually the implementation class) to bind to another class (usually the base class)
        """

        def decorator(cls):
            self.register_singleton(*args, **kwargs)(cls)
            self.bind_to(key, cls)
            return cls

        return decorator

    def require(self, key: Type[T]) -> T:
        if key in self._binding:
            return self.require(self._binding[key])
        elif key in self._container:
            return self._container[key]
        elif key in self._lazy_container:
            # TODO: Lock
            self.register(key, self._lazy_container[key]())
            del self._lazy_container[key]
            return self._container[key]
        elif self._parent is not None:
            return self._parent.require(key)
        else:
            raise KeyError(key)

    def __getitem__(self, key: Type[T]):
        return self.require(key)

    def __contains__(self, key: Type[T]) -> bool:
        if key in self._binding or key in self._container or key in self._lazy_container:
            return True
        elif self._parent is not None:
            return self._parent.contains(key)
        else:
            return False


__all__ = ("Context",)



from typing import TypeVar, Sequence, Any

from nonebot_plugin_pixivbot.data.pixiv_repo import PixivRepo
from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.handler.interceptor.permission_interceptor import SuperuserInterceptor
from nonebot_plugin_pixivbot.handler.utils import post_plain_text
from nonebot_plugin_pixivbot.postman import PostDestination
from .command import SubCommandHandler, CommandHandler

UID = TypeVar("UID")
GID = TypeVar("GID")


@context.require(CommandHandler).sub_command("invalidate_cache")
class InvalidateCacheHandler(SubCommandHandler):
    def __init__(self):
        super().__init__()
        self.pixiv_data_source = context.require(PixivRepo)

        self.add_interceptor(context.require(SuperuserInterceptor))

    @classmethod
    def type(cls) -> str:
        return "invalidate_cache"

    def enabled(self) -> bool:
        return True

    def parse_args(self, args: Sequence[Any], post_dest: PostDestination[UID, GID]) -> dict:
        return {}

    async def actual_handle(self, *, post_dest: PostDestination[UID, GID],
                            silently: bool = False):
        await self.pixiv_data_source.invalidate_cache()
        await post_plain_text(message="ok", post_dest=post_dest)



from typing import TypeVar

from nonebot_plugin_pixivbot.global_context import context
from nonebot_plugin_pixivbot.handler.utils import post_illusts
from nonebot_plugin_pixivbot.postman import PostDestination
from .common import CommonHandler

UID = TypeVar("UID")
GID = TypeVar("GID")


@context.root.register_singleton()
class RandomRecommendedIllustHandler(CommonHandler):
    @classmethod
    def type(cls) -> str:
        return "random_recommended_illust"

    def enabled(self) -> bool:
        return self.conf.pixiv_random_recommended_illust_query_enabled

    async def actual_handle(self, *, count: int = 1,
                            post_dest: PostDestination[UID, GID],
                            silently: bool = False):
        illusts = await self.service.random_recommended_illust(count=count)

        # 记录请求
        self.record_req(post_dest=post_dest, count=count)
        # 记录结果
        if len(illusts) == 1:
            self.record_resp_illust(illusts[0].id, post_dest=post_dest)

        await post_illusts(illusts,
                           header="这是您点的图",
                           post_dest=post_dest)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['nonebot_plugin_pixivbot',
 'nonebot_plugin_pixivbot.data',
 'nonebot_plugin_pixivbot.data.pixiv_repo',
 'nonebot_plugin_pixivbot.data.source',
 'nonebot_plugin_pixivbot.data.source.mongo',
 'nonebot_plugin_pixivbot.data.source.mongo.migration',
 'nonebot_plugin_pixivbot.handler',
 'nonebot_plugin_pixivbot.handler.command',
 'nonebot_plugin_pixivbot.handler.common',
 'nonebot_plugin_pixivbot.handler.interceptor',
 'nonebot_plugin_pixivbot.model',
 'nonebot_plugin_pixivbot.model.old',
 'nonebot_plugin_pixivbot.postman',
 'nonebot_plugin_pixivbot.postman.model',
 'nonebot_plugin_pixivbot.protocol_dep',
 'nonebot_plugin_pixivbot.query',
 'nonebot_plugin_pixivbot.service',
 'nonebot_plugin_pixivbot.utils']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.2.0,<10.0.0',
 'PixivPy-Async>=1.2.14,<2.0.0',
 'lazy>=1.4,<2.0',
 'motor>=3.0.0,<4.0.0',
 'nonebot2>=2.0.0b4,<3.0.0',
 'nonebot_plugin_apscheduler>=0.1.3,<0.2.0',
 'numpy>=1.23.1,<2.0.0']

extras_require = \
{'socks': ['aiohttp-socks>=0.7.1,<0.8.0']}

setup_kwargs = {
    'name': 'nonebot-plugin-pixivbot',
    'version': '1.1.0a2.post1',
    'description': 'Nonebot Plugin PixivBot',
    'long_description': 'nonebot_plugin_pixivbot\n=====\n\nPixivBot中协议无关的通用部分。\n请使用具体协议版本的插件：\n\n- [nonebot-plugin-pixivbot-onebot-v11 (Onebot V11)](https://github.com/ssttkkl/nonebot-plugin-pixivbot-onebot-v11)\n\n## 配置\n\n最小配置：\n\n```\npixiv_refresh_token=  # 前面获取的REFRESH_TOKEN\npixiv_mongo_conn_url=  # MongoDB连接URL，格式：mongodb://<用户名>:<密码>@<主机>:<端口>/<数据库>\npixiv_mongo_database_name=  # 连接的MongoDB数据库\n```\n\n完整配置（除最小配置出现的配置项以外都是可选项，给出的是默认值）：\n\n```\npixiv_refresh_token=  # 前面获取的REFRESH_TOKEN\npixiv_mongo_conn_url=  # MongoDB连接URL，格式：mongodb://<用户名>:<密码>@<主机>:<端口>/<数据库>\npixiv_mongo_database_name=  # 连接的MongoDB数据库\npixiv_proxy=None  # 代理URL\npixiv_query_timeout=60  # 查询超时（单位：秒）\npixiv_simultaneous_query=8  # 向Pixiv查询的并发数\n\n# 缓存过期时间（单位：秒）\npixiv_download_cache_expires_in = 3600 * 24 * 7\npixiv_illust_detail_cache_expires_in = 3600 * 24 * 7\npixiv_user_detail_cache_expires_in = 3600 * 24 * 7\npixiv_illust_ranking_cache_expires_in = 3600 * 6\npixiv_search_illust_cache_expires_in = 3600 * 24\npixiv_search_user_cache_expires_in = 3600 * 24\npixiv_user_illusts_cache_expires_in = 3600 * 24\npixiv_user_bookmarks_cache_expires_in = 3600 * 24\npixiv_related_illusts_cache_expires_in = 3600 * 24\npixiv_other_cache_expires_in = 3600 * 6\n\npixiv_block_tags=[]  # 当插画含有指定tag时会被过滤\npixiv_block_action=no_image  # 过滤时的动作，可选值：no_image(不显示插画，回复插画信息), completely_block(只回复过滤提示), no_reply(无回复)\n\npixiv_download_quantity=original  # 插画下载品质，可选值：original, square_medium, medium, large\npixiv_download_custom_domain=None  # 使用反向代理下载插画的域名\n\npixiv_compression_enabled=False  # 启用插画压缩\npixiv_compression_max_size=None  # 插画压缩最大尺寸\npixiv_compression_quantity=None  # 插画压缩品质（0到100）\n\npixiv_more_enabled=True  # 启用重复上一次请求（还要）功能\n\npixiv_illust_query_enabled=True  # 启用插画查询（看看图）功能\n\npixiv_tag_translation_enabled=True  # 启用搜索关键字翻译功能\n\npixiv_query_cooldown=0  # 每次查询的冷却时间\npixiv_no_query_cooldown_users=[]  # 在这个列表中的用户不受冷却时间的影响\n\npixiv_ranking_query_enabled=True  # 启用榜单查询（看看榜）功能\npixiv_ranking_default_mode=day  # 默认查询的榜单，可选值：day, week, month, day_male, day_female, week_original, week_rookie, day_manga\npixiv_ranking_default_range=[1, 3]  # 默认查询的榜单范围\npixiv_ranking_fetch_item=150  # 每次从服务器获取的榜单项数（查询的榜单范围必须在这个数目内）\npixiv_ranking_max_item_per_query=5  # 每次榜单查询最多能查询多少项\npixiv_ranking_max_item_per_msg=1  # 榜单查询的回复信息每条包含多少项\n\npixiv_random_illust_query_enabled=True  # 启用关键字插画随机抽选（来张xx图）功能\npixiv_random_illust_method=bookmark_proportion  # 随机抽选方法，下同，可选值：bookmark_proportion(概率与书签数成正比), view_proportion(概率与阅读量成正比), timedelta_proportion(概率与投稿时间和现在的时间差成正比), uniform(相等概率)\npixiv_random_illust_min_bookmark=0  # 过滤掉书签数小于该值的插画，下同\npixiv_random_illust_min_view=0  # 过滤掉阅读量小于该值的插画，下同\npixiv_random_illust_max_page=20  # 每次从服务器获取的查询结果页数，下同\npixiv_random_illust_max_item=500  # 每次从服务器获取的查询结果项数，下同\n\npixiv_random_recommended_illust_query_enabled=True  # 启用推荐插画随机抽选（来张图）功能\npixiv_random_recommended_illust_method=uniform\npixiv_random_recommended_illust_min_bookmark=0\npixiv_random_recommended_illust_min_view=0\npixiv_random_recommended_illust_max_page=40\npixiv_random_recommended_illust_max_item=1000\n\npixiv_random_related_illust_query_enabled = True  # 启用关联插画随机抽选（不够色）功能\npixiv_random_related_illust_method = "bookmark_proportion"\npixiv_random_related_illust_min_bookmark = 0\npixiv_random_related_illust_min_view = 0\npixiv_random_related_illust_max_page = 4\npixiv_random_related_illust_max_item = 100\n\npixiv_random_user_illust_query_enabled=True  # 启用用户插画随机抽选（来张xx老师的图）功能\npixiv_random_user_illust_method=timedelta_proportion\npixiv_random_user_illust_min_bookmark=0\npixiv_random_user_illust_min_view=0\npixiv_random_user_illust_max_page=2147483647\npixiv_random_user_illust_max_item=2147483647\n\npixiv_random_bookmark_query_enabled=True  # 启用用户书签随机抽选（来张私家车）功能\npixiv_random_bookmark_user_id=0  # 当QQ用户未绑定Pixiv账号时，从该Pixiv账号的书签内抽选\npixiv_random_bookmark_method=uniform\npixiv_random_bookmark_min_bookmark=0\npixiv_random_bookmark_min_view=0\npixiv_random_bookmark_max_page=2147483647\npixiv_random_bookmark_max_item=2147483647\n\npixiv_poke_action=random_recommended_illust  # 戳一戳的功能，可选值：空, ranking, random_recommended_illust, random_bookmark\n```\n\n## Special Thanks\n\n[Mikubill/pixivpy-async](https://github.com/Mikubill/pixivpy-async)\n\n[nonebot/nonebot2](https://github.com/nonebot/nonebot2)\n\n## LICENSE\n\n```\nMIT License\n\nCopyright (c) 2021 ssttkkl\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.\n\n```\n',
    'author': 'ssttkkl',
    'author_email': 'huang.wen.long@hotmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ssttkkl/nonebot-plugin-pixivbot',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from typing import TypeVar, Generic, Optional

UID = TypeVar("UID")
GID = TypeVar("GID")


@dataclass(frozen=True)
class UserIdentifier(Generic[UID]):
    adapter: str
    user_id: UID

    def __str__(self):
        return f"{self.adapter}:{self.user_id}"


@dataclass(frozen=True)
class PostIdentifier(Generic[UID, GID]):
    adapter: str
    user_id: Optional[UID]
    group_id: Optional[GID]

    def __str__(self):
        return f"{self.adapter}:{self.user_id}:{self.group_id}"

    def __post_init__(self):
        if not self.user_id and not self.group_id:
            raise ValueError("at least one of user_id and group_id should be not None")



from .post_destination import PostDestination, PostDestinationFactory, PostDestinationFactoryManager
from .postman import Postman, PostmanManager

__all__ = ("Postman", "PostmanManager",
           "PostDestination", "PostDestinationFactory", "PostDestinationFactoryManager")
